@@ -12,14 +12,17 @@
 
 -module(couch).
 
--compile(export_all).
+-export([
+    start/0,
+    stop/0,
+    restart/0
+]).
 
 
 deps() ->
     [
         sasl,
         inets,
-        os_mon,
         crypto,
         public_key,
         ssl,
