@@ -408,7 +408,7 @@ parse_rep_db({Props}, Proxy, Options) ->
     {BinHeaders} = get_value(<<"headers">>, Props, {[]}),
     Headers = lists:ukeysort(1, [{?b2l(K), ?b2l(V)} || {K, V} <- BinHeaders]),
     DefaultHeaders = (#httpdb{})#httpdb.headers,
-    #httpdb{
+    HttpDb = #httpdb{
         url = Url,
         auth_props = AuthProps,
         headers = lists:ukeymerge(1, Headers, DefaultHeaders),
@@ -419,7 +419,8 @@ parse_rep_db({Props}, Proxy, Options) ->
         http_connections = get_value(http_connections, Options),
         retries = get_value(retries, Options),
         proxy_url = ProxyURL
-    };
+    },
+    couch_replicator_utils:normalize_basic_auth(HttpDb);
 
 parse_rep_db(<<"http://", _/binary>> = Url, Proxy, Options) ->
     parse_rep_db({[{<<"url">>, Url}]}, Proxy, Options);
@@ -455,26 +456,26 @@ maybe_add_trailing_slash(Url) ->
 make_options(Props) ->
     Options0 = lists:ukeysort(1, convert_options(Props)),
     Options = check_options(Options0),
-    DefWorkers = config:get("replicator", "worker_processes", "4"),
-    DefBatchSize = config:get("replicator", "worker_batch_size", "500"),
-    DefConns = config:get("replicator", "http_connections", "20"),
-    DefTimeout = config:get("replicator", "connection_timeout", "30000"),
-    DefRetries = config:get("replicator", "retries_per_request", "5"),
-    UseCheckpoints = config:get("replicator", "use_checkpoints", "true"),
-    DefCheckpointInterval = config:get("replicator", "checkpoint_interval",
-        "30000"),
+    DefWorkers = config:get_integer("replicator", "worker_processes", 4),
+    DefBatchSize = config:get_integer("replicator", "worker_batch_size", 500),
+    DefConns = config:get_integer("replicator", "http_connections", 20),
+    DefTimeout = config:get_integer("replicator", "connection_timeout", 30000),
+    DefRetries = config:get_integer("replicator", "retries_per_request", 5),
+    UseCheckpoints = config:get_boolean("replicator", "use_checkpoints", true),
+    DefCheckpointInterval = config:get_integer("replicator",
+        "checkpoint_interval", 30000),
     {ok, DefSocketOptions} = couch_util:parse_term(
         config:get("replicator", "socket_options",
             "[{keepalive, true}, {nodelay, false}]")),
     lists:ukeymerge(1, Options, lists:keysort(1, [
-        {connection_timeout, list_to_integer(DefTimeout)},
-        {retries, list_to_integer(DefRetries)},
-        {http_connections, list_to_integer(DefConns)},
+        {connection_timeout, DefTimeout},
+        {retries, DefRetries},
+        {http_connections, DefConns},
         {socket_options, DefSocketOptions},
-        {worker_batch_size, list_to_integer(DefBatchSize)},
-        {worker_processes, list_to_integer(DefWorkers)},
-        {use_checkpoints, list_to_existing_atom(UseCheckpoints)},
-        {checkpoint_interval, list_to_integer(DefCheckpointInterval)}
+        {worker_batch_size, DefBatchSize},
+        {worker_processes, DefWorkers},
+        {use_checkpoints, UseCheckpoints},
+        {checkpoint_interval, DefCheckpointInterval}
     ])).
 
 
@@ -570,8 +571,7 @@ parse_proxy_params(ProxyUrl) ->
         password = Passwd,
         protocol = Protocol
     } = ibrowse_lib:parse_url(ProxyUrl),
-    [
-        {proxy_protocol, Protocol},
+    Params = [
         {proxy_host, Host},
         {proxy_port, Port}
     ] ++ case is_list(User) andalso is_list(Passwd) of
@@ -579,21 +579,38 @@ parse_proxy_params(ProxyUrl) ->
             [];
         true ->
             [{proxy_user, User}, {proxy_password, Passwd}]
-        end.
+    end,
+    case Protocol of
+        socks5 ->
+            [proxy_to_socks5(Param) || Param <- Params];
+        _ ->
+            Params
+    end.
+
+
+-spec proxy_to_socks5({atom(), string()}) -> {atom(), string()}.
+proxy_to_socks5({proxy_host, Val}) ->
+    {socks5_host, Val};
+proxy_to_socks5({proxy_port, Val}) ->
+    {socks5_port, Val};
+proxy_to_socks5({proxy_user, Val}) ->
+    {socks5_user, Val};
+proxy_to_socks5({proxy_password, Val}) ->
+    {socks5_password, Val}.
 
 
 -spec ssl_params([_]) -> [_].
 ssl_params(Url) ->
     case ibrowse_lib:parse_url(Url) of
     #url{protocol = https} ->
-        Depth = list_to_integer(
-            config:get("replicator", "ssl_certificate_max_depth", "3")
-        ),
-        VerifyCerts = config:get("replicator", "verify_ssl_certificates"),
+        Depth = config:get_integer("replicator",
+            "ssl_certificate_max_depth", 3),
+        VerifyCerts = config:get_boolean("replicator",
+            "verify_ssl_certificates", false),
         CertFile = config:get("replicator", "cert_file", undefined),
         KeyFile = config:get("replicator", "key_file", undefined),
         Password = config:get("replicator", "password", undefined),
-        SslOpts = [{depth, Depth} | ssl_verify_options(VerifyCerts =:= "true")],
+        SslOpts = [{depth, Depth} | ssl_verify_options(VerifyCerts)],
         SslOpts1 = case CertFile /= undefined andalso KeyFile /= undefined of
             true ->
                 case Password of
@@ -789,6 +806,21 @@ check_strip_credentials_test() ->
     ]].
 
 
+parse_proxy_params_test() ->
+    ?assertEqual([
+        {proxy_host, "foo.com"},
+        {proxy_port, 443},
+        {proxy_user, "u"},
+        {proxy_password, "p"}
+    ], parse_proxy_params("https://u:p@foo.com")),
+    ?assertEqual([
+        {socks5_host, "foo.com"},
+        {socks5_port, 1080},
+        {socks5_user, "u"},
+        {socks5_password, "p"}
+    ], parse_proxy_params("socks5://u:p@foo.com")).
+
+
 setup() ->
     DbName = ?tempdb(),
     {ok, Db} = couch_db:create(DbName, [?ADMIN_CTX]),
