@@ -29,7 +29,7 @@ function createTest (db) {
   return (client) => {
 
     const waitTime = client.globals.maxWaitTime;
-    const baseUrl = client.globals.test_settings.launch_url;
+    const baseUrl = client.options.launch_url;
 
     client
       .createDatabase(db)
@@ -37,7 +37,6 @@ function createTest (db) {
 
       .url(baseUrl + '#/_all_dbs')
       .clickWhenVisible(`.fauxton-table-list a[href="database/${encodeURIComponent(db)}/_all_docs"]`)
-      .waitForElementNotPresent('#global-notifications .fonticon-cancel', waitTime, false)
 
       .waitForElementVisible('.faux-header__doc-header-title', waitTime, false)
       .waitForElementVisible('.no-results-screen', waitTime, false)
