@@ -18,13 +18,16 @@ module.exports = {
   // a bug fix for where the Ace Editor threw a JS error preventing the subsequent page from loading]
   'Navigate to a database that does not exist' : function (client) {
     const waitTime = client.globals.maxWaitTime,
-          baseUrl = client.globals.test_settings.launch_url;
+          baseUrl = client.options.launch_url;
 
     client
       .loginToGUI()
       .url(baseUrl + '/#/database/does-not-exist/_all_docs')
-      .waitForElementPresent('.global-notification', waitTime, false)
-      .assert.containsText('.global-notification', 'does not exist')
+      .waitForElementVisible('.Toastify__toast-container .Toastify__toast--error', waitTime, false)
+      // We wait for the first toasts to be cleared
+      .pause(1000)
+      .waitForElementVisible('.Toastify__toast-container .Toastify__toast--error', waitTime, false)
+      .assert.containsText('.Toastify__toast-container .Toastify__toast--error .Toastify__toast-body', 'does not exist')
       .verify.urlEquals(baseUrl + '/#');
   }
 };
