@@ -23,6 +23,7 @@
 -export([start/2]).
 -export([set_expect/2]).
 -export([delete_expect/4]).
+-export([list_expects/2]).
 -export([get_history/1]).
 -export([wait/6]).
 -export([reset/1]).
@@ -47,11 +48,7 @@
 %%% Definitions
 %%%============================================================================
 
--ifdef(non_namespaced_types). % For older Erlang versions
--type meck_dict() :: dict().
--else.
 -type meck_dict() :: dict:dict().
--endif.
 
 -record(state, {mod :: atom(),
                 can_expect :: any | [{Mod::atom(), Ari::byte()}],
@@ -124,6 +121,11 @@ set_expect(Mod, Expect) ->
 delete_expect(Mod, Func, Ari, Force) ->
     gen_server(call, Mod, {delete_expect, Func, Ari, Force}).
 
+-spec list_expects(Mod::atom(), ExcludePassthrough::boolean()) ->
+    [{Mod::atom(), Func::atom(), Ari::byte()}].
+list_expects(Mod, ExcludePassthrough) ->
+    gen_server(call, Mod, {list_expects, ExcludePassthrough}).
+
 -spec add_history_exception(
         Mod::atom(), CallerPid::pid(), Func::atom(), Args::[any()],
         {Class::error|exit|throw, Reason::any(), StackTrace::any()}) ->
@@ -191,8 +193,24 @@ stop(Mod) ->
 %%% gen_server callbacks
 %%%============================================================================
 
+%% @hidden
+validate_options([]) -> ok;
+validate_options([no_link|Options]) -> validate_options(Options);
+validate_options([spawn_opt|Options]) -> validate_options(Options);
+validate_options([unstick|Options]) -> validate_options(Options);
+validate_options([no_passthrough_cover|Options]) -> validate_options(Options);
+validate_options([merge_expects|Options]) -> validate_options(Options);
+validate_options([enable_on_load|Options]) -> validate_options(Options);
+validate_options([passthrough|Options]) -> validate_options(Options);
+validate_options([no_history|Options]) -> validate_options(Options);
+validate_options([non_strict|Options]) -> validate_options(Options);
+validate_options([stub_all|Options]) -> validate_options(Options);
+validate_options([{stub_all, _}|Options]) -> validate_options(Options);
+validate_options([UnknownOption|_]) -> erlang:error({bad_arg, UnknownOption}).
+
 %% @hidden
 init([Mod, Options]) ->
+    validate_options(Options),
     Exports = normal_exports(Mod),
     WasSticky = case proplists:get_bool(unstick, Options) of
         true -> {module, Mod} = code:ensure_loaded(Mod),
@@ -203,7 +221,7 @@ init([Mod, Options]) ->
     MergeExpects = proplists:get_bool(merge_expects, Options),
     EnableOnLoad = proplists:get_bool(enable_on_load, Options),
     Passthrough = proplists:get_bool(passthrough, Options),
-    Original = backup_original(Mod, NoPassCover, EnableOnLoad),
+    Original = backup_original(Mod, Passthrough, NoPassCover, EnableOnLoad),
     NoHistory = proplists:get_bool(no_history, Options),
     History = if NoHistory -> undefined; true -> [] end,
     CanExpect = resolve_can_expect(Mod, Exports, Options),
@@ -222,7 +240,7 @@ init([Mod, Options]) ->
                     history = History}}
     catch
         exit:{error_loading_module, Mod, sticky_directory} ->
-            {stop, module_is_sticky}
+            {stop, {module_is_sticky, Mod}}
     end.
 
 %% @hidden
@@ -230,27 +248,46 @@ handle_call({get_result_spec, Func, Args}, _From, S) ->
     {ResultSpec, NewExpects} = do_get_result_spec(S#state.expects, Func, Args),
     {reply, ResultSpec, S#state{expects = NewExpects}};
 handle_call({set_expect, Expect}, From,
-            S = #state{mod = Mod, expects = Expects, merge_expects = MergeExpects}) ->
+            S = #state{mod = Mod, expects = Expects, passthrough = Passthrough,
+                       merge_expects = MergeExpects, can_expect = CanExpect}) ->
     check_if_being_reloaded(S),
     FuncAri = {Func, Ari} = meck_expect:func_ari(Expect),
     case validate_expect(Mod, Func, Ari, S#state.can_expect) of
         ok ->
-            {NewExpects, CompilerPid} = store_expect(Mod, FuncAri, Expect,
-                                                     Expects, MergeExpects),
-            {noreply, S#state{expects = NewExpects,
-                              reload = {CompilerPid, From}}};
+            case store_expect(Mod, FuncAri, Expect, Expects,
+                              MergeExpects, Passthrough, CanExpect) of
+                {no_compile, NewExpects} ->
+                    {reply, ok, S#state{expects = NewExpects}};
+                {CompilerPid, NewExpects} ->
+                    {noreply, S#state{expects = NewExpects,
+                                      reload = {CompilerPid, From}}}
+            end;
         {error, Reason} ->
             {reply, {error, Reason}, S}
     end;
 handle_call({delete_expect, Func, Ari, Force}, From,
             S = #state{mod = Mod, expects = Expects,
-                       passthrough = PassThrough}) ->
+                       passthrough = PassThrough, can_expect = CanExpect}) ->
     check_if_being_reloaded(S),
     ErasePassThrough = Force orelse (not PassThrough),
-    {NewExpects, CompilerPid} =
-        do_delete_expect(Mod, {Func, Ari}, Expects, ErasePassThrough),
-    {noreply, S#state{expects = NewExpects,
-                      reload = {CompilerPid, From}}};
+    case do_delete_expect(Mod, {Func, Ari}, Expects, ErasePassThrough,
+                          PassThrough, CanExpect) of
+        {no_compile, NewExpects} ->
+            {reply, ok, S#state{expects = NewExpects}};
+        {CompilerPid, NewExpects} ->
+            {noreply, S#state{expects = NewExpects, reload = {CompilerPid, From}}}
+    end;
+handle_call({list_expects, ExcludePassthrough}, _From, S = #state{mod = Mod, expects = Expects}) ->
+    Result =
+        case ExcludePassthrough of
+            false ->
+                [{Mod, Func, Ari} || {Func, Ari} <- dict:fetch_keys(Expects)];
+            true ->
+                [{Mod, Func, Ari} ||
+                    {{Func, Ari}, Expect} <- dict:to_list(Expects),
+                    not meck_expect:is_passthrough(Expect)]
+        end,
+    {reply, Result, S};
 handle_call(get_history, _From, S = #state{history = undefined}) ->
     {reply, [], S};
 handle_call(get_history, _From, S) ->
@@ -283,20 +320,13 @@ handle_cast(invalidate, S) ->
     {noreply, S#state{valid = false}};
 handle_cast({add_history, HistoryRecord}, S = #state{history = undefined,
                                                      trackers = Trackers}) ->
-    UpdTracker = update_trackers(HistoryRecord, Trackers),
-    {noreply, S#state{trackers = UpdTracker}};
+    UpdTrackers = update_trackers(HistoryRecord, Trackers),
+    {noreply, S#state{trackers = UpdTrackers}};
 handle_cast({add_history, HistoryRecord}, S = #state{history = History,
-                                                     trackers = Trackers,
-                                                     reload = Reload}) ->
-    case Reload of
-        undefined ->
-            UpdTrackers = update_trackers(HistoryRecord, Trackers),
-            {noreply, S#state{history = [HistoryRecord | History],
-                              trackers = UpdTrackers}};
-        _ ->
-            % Skip Item if the mocked module compiler is running.
-            {noreply, S}
-    end;
+                                                     trackers = Trackers}) ->
+    UpdTrackers = update_trackers(HistoryRecord, Trackers),
+    {noreply, S#state{history = [HistoryRecord | History],
+                      trackers = UpdTrackers}};
 handle_cast(_Msg, S)  ->
     {noreply, S}.
 
@@ -346,20 +376,20 @@ expect_type(Mod, Func, Ari) ->
         false -> normal
     end.
 
--spec backup_original(Mod::atom(), NoPassCover::boolean(), EnableOnLoad::boolean()) ->
+-spec backup_original(Mod::atom(), Passthrough::boolean(), NoPassCover::boolean(), EnableOnLoad::boolean()) ->
     {Cover:: false |
              {File::string(), Data::string(), CompiledOptions::[any()]},
      Binary:: no_binary |
               no_passthrough_cover |
               binary()}.
-backup_original(Mod, NoPassCover, EnableOnLoad) ->
+backup_original(Mod, Passthrough, NoPassCover, EnableOnLoad) ->
     Cover = get_cover_state(Mod),
     try
         Forms0 = meck_code:abstract_code(meck_code:beam_file(Mod)),
         Forms = meck_code:enable_on_load(Forms0, EnableOnLoad),
         NewName = meck_util:original_name(Mod),
         CompileOpts = [debug_info | meck_code:compile_options(meck_code:beam_file(Mod))],
-        Renamed = meck_code:rename_module(Forms, NewName),
+        Renamed = meck_code:rename_module(Forms, Mod, NewName),
         Binary = meck_code:compile_and_load_forms(Renamed, CompileOpts),
 
         %% At this point we care about `Binary' if and only if we want
@@ -384,15 +414,18 @@ backup_original(Mod, NoPassCover, EnableOnLoad) ->
             (Cover == false) orelse NoPassCover ->
                 no_passthrough_cover;
             true ->
-                meck_cover:compile_beam(NewName, Binary),
+                _ = meck_cover:compile_beam(NewName, Binary),
                 Binary
         end,
         {Cover, Binary2}
     catch
         throw:{object_code_not_found, _Module} ->
             {Cover, no_binary}; % TODO: What to do here?
-        throw:no_abstract_code                 ->
-            {Cover, no_binary} % TODO: What to do here?
+        throw:no_abstract_code ->
+            case Passthrough of
+                true  -> exit({abstract_code_not_found, Mod});
+                false -> {Cover, no_binary}
+            end
     end.
 
 -spec get_cover_state(Mod::atom()) ->
@@ -460,6 +493,8 @@ gen_server(Func, Mod, Msg) ->
 -spec check_if_being_reloaded(#state{}) -> ok.
 check_if_being_reloaded(#state{reload = undefined}) ->
     ok;
+check_if_being_reloaded(#state{passthrough = true}) ->
+    ok;
 check_if_being_reloaded(_S) ->
     erlang:error(concurrent_reload).
 
@@ -494,25 +529,39 @@ validate_expect(Mod, Func, Ari, CanExpect) ->
     end.
 
 -spec store_expect(Mod::atom(), meck_expect:func_ari(),
-                   meck_expect:expect(), Expects::meck_dict(), boolean()) ->
-        {NewExpects::meck_dict(), CompilerPid::pid()}.
-store_expect(Mod, FuncAri, Expect, Expects, true) ->
+                   meck_expect:expect(), Expects::meck_dict(),
+                   MergeExpects::boolean(), Passthrough::boolean(),
+                   CanExpect::term()) ->
+        {CompilerPidOrNoCompile::no_compile | pid(), NewExpects::meck_dict()}.
+store_expect(Mod, FuncAri, Expect, Expects, true, PassThrough, CanExpect) ->
     NewExpects = case dict:is_key(FuncAri, Expects) of
         true ->
             {FuncAri, ExistingClauses} = dict:fetch(FuncAri, Expects),
             {FuncAri, NewClauses} = Expect,
-            dict:store(FuncAri, {FuncAri, ExistingClauses ++ NewClauses}, Expects);
+            ToStore = case PassThrough of
+                          false ->
+                              ExistingClauses ++ NewClauses;
+                          true ->
+                              RevExistingClauses = lists:reverse(ExistingClauses),
+                              [PassthroughClause | OldClauses] = RevExistingClauses,
+                              lists:reverse(OldClauses,
+                                            NewClauses ++ [PassthroughClause])
+                      end,
+            dict:store(FuncAri, {FuncAri, ToStore}, Expects);
         false -> dict:store(FuncAri, Expect, Expects)
     end,
-    compile_expects(Mod, NewExpects);
-store_expect(Mod, FuncAri, Expect, Expects, false) ->
+    {compile_expects_if_needed(Mod, NewExpects, PassThrough, CanExpect),
+     NewExpects};
+store_expect(Mod, FuncAri, Expect, Expects, false, PassThrough, CanExpect) ->
     NewExpects =  dict:store(FuncAri, Expect, Expects),
-    compile_expects(Mod, NewExpects).
+    {compile_expects_if_needed(Mod, NewExpects, PassThrough, CanExpect),
+     NewExpects}.
 
 -spec do_delete_expect(Mod::atom(), meck_expect:func_ari(),
-                       Expects::meck_dict(), ErasePassThrough::boolean()) ->
-        {NewExpects::meck_dict(), CompilerPid::pid()}.
-do_delete_expect(Mod, FuncAri, Expects, ErasePassThrough) ->
+                       Expects::meck_dict(), ErasePassThrough::boolean(),
+                       Passthrough::boolean(), CanExpect::term()) ->
+        {CompilerPid::no_compile | pid(), NewExpects::meck_dict()}.
+do_delete_expect(Mod, FuncAri, Expects, ErasePassThrough, Passthrough, CanExpect) ->
     NewExpects = case ErasePassThrough of
                      true  ->
                          dict:erase(FuncAri, Expects);
@@ -521,31 +570,38 @@ do_delete_expect(Mod, FuncAri, Expects, ErasePassThrough) ->
                                     meck_expect:new_passthrough(FuncAri),
                                     Expects)
                  end,
-    compile_expects(Mod, NewExpects).
+    {compile_expects_if_needed(Mod, NewExpects, Passthrough, CanExpect),
+     NewExpects}.
+
+-spec compile_expects_if_needed(Mod::atom(), Expects::meck_dict(),
+                                Passthrough::boolean(), CanExpect::term()) ->
+        CompilerPidOrNoCompile::no_compile | pid().
+compile_expects_if_needed(_Mod, _Expects, true, CanExpect) when CanExpect =/= any ->
+    no_compile;
+compile_expects_if_needed(Mod, Expects, _, _) ->
+    compile_expects(Mod, Expects).
 
 -spec compile_expects(Mod::atom(), Expects::meck_dict()) ->
-        {NewExpects::meck_dict(), CompilerPid::pid()}.
+        CompilerPid::pid().
 compile_expects(Mod, Expects) ->
     %% If the recompilation is made by the server that executes a module
     %% no module that is called from meck_code:compile_and_load_forms/2
     %% can be mocked by meck.
-    CompilerPid =
-        erlang:spawn_link(fun() ->
-                                  Forms = meck_code_gen:to_forms(Mod, Expects),
-                                  meck_code:compile_and_load_forms(Forms)
-                          end),
-    {Expects, CompilerPid}.
+    erlang:spawn_link(fun() ->
+                              Forms = meck_code_gen:to_forms(Mod, Expects),
+                              meck_code:compile_and_load_forms(Forms)
+                      end).
 
 restore_original(Mod, {false, _Bin}, WasSticky, _BackupCover) ->
     restick_original(Mod, WasSticky),
     ok;
 restore_original(Mod, {{File, OriginalCover, Options}, _Bin}, WasSticky, BackupCover) ->
-    case filename:extension(File) of
-        ".erl" ->
-            {ok, Mod} = cover:compile_module(File, Options);
-        ".beam" ->
-            cover:compile_beam(File)
-    end,
+    {ok, Mod} = case filename:extension(File) of
+                    ".erl" ->
+                        cover:compile_module(File, Options);
+                    ".beam" ->
+                        cover:compile_beam(File)
+                end,
     restick_original(Mod, WasSticky),
     if BackupCover =/= undefined ->
         %% Import the cover data for `<name>_meck_original' but since it was
